-- KikiInventory Panel


euroFormat=(tonumber("1,000")==1); -- redundant but harmless


InventoryPanel = class(Turbine.UI.Lotro.Window);

function InventoryPanel:BuildMenu()
	self.characterSelectionMenu = Turbine.UI.ContextMenu();
	 
	local menuItems = self.characterSelectionMenu:GetItems();

	-- Iteration sur tous les noms de personnages prsent dans la configuration
	-- Comme tous les personnages ont un bag, on itere simplement sur le bag
	local tblUsr =  {} ;
	for usr,bag in pairs(gBag.data) do
		table.insert(tblUsr,usr);
	end
	table.sort(tblUsr);
		
	local mi = Turbine.UI.MenuItem(gLang.ALLCHARACTERS);
	mi.Click = function( sender, args)
		self:SelectCharacter(nil);
	end
	menuItems:Add(mi);
	for i,usr in pairs(tblUsr) do
		mi = Turbine.UI.MenuItem(usr);
		mi.Click = function( sender, args)
			self:SelectCharacter(usr); 
		end
		menuItems:Add(mi);
	end
	
	-- Separateur
	mi = Turbine.UI.MenuItem(" ",false);
	menuItems:Add(mi);
	
	-- Option de suppression du personnage actuel
	self.deleteCurrentControl = Turbine.UI.MenuItem(gLang.MENUDELETE,false);
	self.deleteCurrentControl.Click = function( sender, args)
		self:DeleteCurrentCharacter(); 
	end
	menuItems:Add(self.deleteCurrentControl);
end
--
-- LAYOUT DE LA FENETRE
--
-- Layout : Ajoute la gestion de la slection d'un personnage ou tous les personnages
function InventoryPanel:AddCharacterSelection()

	-- menu contextuel 
	self:BuildMenu();
	
	-- Ajoute le bouton de selection du personnage
	local btn = Turbine.UI.Lotro.Button();
    btn:SetPosition( 90 , 45 );
    btn:SetSize( 150, 32 );
    btn:SetParent( self );
    btn:SetFont(Turbine.UI.Lotro.Font.Verdana12);
	-- Sur click, on affiche le menu de selection
	btn.Click = function()
		-- Si un personnage est selectionn, active l'option de suppression
		if self.characterSelectionName ~= nil and self.characterSelectionName ~= gPlayerName then
			self.deleteCurrentControl:SetText(gLang.MENUDELETE.." "..self.characterSelectionName);
			self.deleteCurrentControl:SetEnabled(true);
		else
			self.deleteCurrentControl:SetText(gLang.MENUDELETENA);
			self.deleteCurrentControl:SetEnabled(false);
		end
		self.characterSelectionMenu:ShowMenu();
	end
	
	-- Association du bouton
	self.characterSelectionButton = btn; 

	-- Ajoute le label "personnage"
	local lbl = Turbine.UI.Label();
    lbl:SetPosition( btn:GetLeft()-85 , btn:GetTop() );
    lbl:SetSize( 80, btn:GetHeight());
    lbl:SetParent( self );
    lbl:SetFont(Turbine.UI.Lotro.Font.Verdana14);
	lbl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
    lbl:SetText(gLang.SELCHAR);
end

-- Classe de bouton de type selection du storage
StorageButton = class(Turbine.UI.Control);
function StorageButton:Constructor(parent,type,tip)
	Turbine.UI.Control.Constructor(self);
	self:SetParent(parent);
	self:SetSize(32,32);
	self:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.MouseClick = function()
		parent:SelectStorage(type);
	end
	Kiki.Commons.TooltipAttach(self,tip);
end
-- Layout : Ajoute la gestion de la slection d'un personnage ou tous les personnages
function InventoryPanel:AddStorageSelection()
	self.selectBag = StorageButton(self,"bag",gLang.SELBAG);
	self.selectBag:SetPosition(self:GetWidth()-160,40);
	self.selectVault = StorageButton(self,"vault",gLang.SELVAULT);
	self.selectVault:SetPosition(self:GetWidth()-130,40);
	self.selectShared = StorageButton(self,"shared",gLang.SELSHARED);
	self.selectShared:SetPosition(self:GetWidth()-100,40);
	self.selectWallet = StorageButton(self,"wallet",gLang.SELWALLET);
	self.selectWallet:SetPosition(self:GetWidth()-70,40);
	self.selectAll = StorageButton(self,nil,gLang.SELALL);
	self.selectAll:SetPosition(self:GetWidth()-45,40);
end

-- Layout : Ajoute le filtre sur le nom des items
function InventoryPanel:AddItemFilter()

	local stxt=Turbine.UI.Lotro.TextBox();
	stxt:SetParent(self);
	stxt:SetPosition(90,75);
	stxt:SetSize(150,20);
	stxt:SetBackColor(Turbine.UI.Color(.25,.35,.55));
	stxt:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	stxt:SetFont(self.font);
	stxt:SetForeColor(self.fontColor);
	stxt:SetOutlineColor(Turbine.UI.Color(0.5,0,0));
	stxt:SetFontStyle(Turbine.UI.FontStyle.Outline);
	stxt:SetWantsUpdates(false);
	stxt.Text="";
	stxt:SetMultiline(false);
	stxt.FocusGained=function()
		stxt:SetWantsUpdates(true);
	end
	stxt.FocusLost=function()
		stxt:SetWantsUpdates(false);
	end
	stxt.Update=function()
		self:SelectText(stxt:GetText());
	end
	Kiki.Commons.TooltipAttach(stxt,gLang.TTFILTER);
	-- Ajoute le label "filtre"
	local lbl = Turbine.UI.Label();
    lbl:SetPosition( stxt:GetLeft()-85 , stxt:GetTop() );
    lbl:SetSize( 80, stxt:GetHeight());
    lbl:SetParent( self );
    lbl:SetFont(Turbine.UI.Lotro.Font.Verdana14);
	lbl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
    lbl:SetText(gLang.SEARCH);
end


-- Layout : Zone des elements  afficher
function InventoryPanel:AddItemList()
	self.ItemBg =Turbine.UI.Control();
	self.ItemBg:SetParent(self);
	self.ItemBg:SetPosition(10,100);
	self.ItemBg:SetSize(self:GetWidth()-20,self:GetHeight()-140);
	-- self.ItemBg:SetBackColor(Turbine.UI.Color(0,0,0)); 
	self.ItemBg.List=Turbine.UI.ListBox();
	self.ItemBg.List:SetParent(self.ItemBg);
	self.ItemBg.List:SetPosition(2,1);
	self.ItemBg.List:SetSize(self.ItemBg:GetWidth()-4,self.ItemBg:GetHeight()-2);
	self.ItemBg.List:SetBackColor(Turbine.UI.Color(0,0,0));
	self.ItemBg.List.VScrollBar=Turbine.UI.Lotro.ScrollBar();
	self.ItemBg.List.VScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.ItemBg.List.VScrollBar:SetParent(self.ItemBg.List);
	self.ItemBg.List.VScrollBar:SetBackColor(backColor);
	self.ItemBg.List.VScrollBar:SetPosition(self.ItemBg:GetWidth()-16,0)
	self.ItemBg.List.VScrollBar:SetWidth(12);
	self.ItemBg.List.VScrollBar:SetHeight(self.ItemBg:GetHeight()-2);
	self.ItemBg.List:SetAllowDrop( false );
	self.ItemBg.List:SetVerticalScrollBar(self.ItemBg.List.VScrollBar);
end


function InventoryPanel:SelectText(t)
	if self.selectedText ~= t then
		if #t == 0 then
			self.selectedText= nil;
		else
			self.selectedText = t;
		end
		self:Load();
	end
end
function InventoryPanel:SelectStorage(s)
	if s=="bag" then
		self.selectBag:SetBackground("Kiki/KikiInventory/Resources/Inventory.tga");
	else
		self.selectBag:SetBackground("Kiki/KikiInventory/Resources/Inventory_grey.tga");
	end
	if s=="vault" then
		self.selectVault:SetBackground("Kiki/KikiInventory/Resources/Vault.tga");
	else
		self.selectVault:SetBackground("Kiki/KikiInventory/Resources/Vault_grey.tga");
	end
	if s=="shared" then
		self.selectShared:SetBackground("Kiki/KikiInventory/Resources/SharedStorage.tga");
	else
		self.selectShared:SetBackground("Kiki/KikiInventory/Resources/SharedStorage_grey.tga");
	end
	if s=="wallet" then
		self.selectWallet:SetBackground("Kiki/KikiInventory/Resources/Wallet.tga");
	else
		self.selectWallet:SetBackground("Kiki/KikiInventory/Resources/Wallet_grey.tga");
	end
	if s==nil then
		self.selectAll:SetBackground("Kiki/KikiInventory/Resources/Star.tga");
	else
		self.selectAll:SetBackground("Kiki/KikiInventory/Resources/Star_grey.tga");
	end
	self.selectedStorage = s;
	-- Filtre modifi, modifie la liste affiche
	self:Load();
end

-- Filtre: Positionne un filtre de personnage
-- c est nil ou le nom  filtrer 
-- text, si prsent, est le texte  afficher dans le bouton
function InventoryPanel:SelectCharacter(c,text)
	-- Si le filtre n'est pas modifi, on ne fait rien
	if c == self.characterSelectionName then
		return;
	end
	-- Positionne le nouvel filtre de personnage
	self.characterSelectionName = c;
	if text ~= nil then
		self.characterSelectionButton:SetText(text);
	elseif c == nil then
		self.characterSelectionButton:SetText(gLang.ALLCHARACTERS);		
	else
		self.characterSelectionButton:SetText(c);		
	end
	-- Filtre modifi, modifie la liste affiche
	self:Load();
	
end



-- Construi le controle d'un item dans la liste de l'inventaire
function InventoryPanel:BuildControlItem(item)
	-- Construction de l'element
	local ctl = Turbine.UI.Control();
	ctl:SetSize(self.ItemBg.List:GetWidth()-18,36);
	-- ctl:SetBackColor(Turbine.UI.Color(0.2,0.2,0.2));

	local border = Turbine.UI.Control();
	border:SetSize(ctl:GetWidth(),ctl:GetHeight()-2);
	border:SetPosition(0,1);
	border:SetBackColor(Turbine.UI.Color(0.5,0.5,0.5));
	border:SetParent(ctl);

	local icon = Turbine.UI.Control();
	icon:SetSize(32,32);
	icon:SetPosition(1,1);
	if item.backId ~= nil then
		icon:SetBackground(item.backId);
	else
		icon:SetBackColor(Turbine.UI.Color(0,0,0));
	end
	icon:SetParent(border);
	-- icon:SetBlendMode(Turbine.UI.BlendMode.Overlay);

	local iconImage = Turbine.UI.Control();
	iconImage:SetSize(32,32);
	iconImage:SetPosition(0,0);
	iconImage:SetBackground(item.iconId);
	iconImage:SetParent(icon);
	iconImage:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	

	local qty = Turbine.UI.Label();
	qty:SetParent(iconImage);
	qty:SetPosition(0,11);
	qty:SetSize(30,20);
	qty:SetFont(Turbine.UI.Lotro.Font.Verdana10);
	qty:SetOutlineColor(Turbine.UI.Color(0.5,0,0));
	qty:SetFontStyle(Turbine.UI.FontStyle.Outline);
	qty:SetTextAlignment(Turbine.UI.ContentAlignment.BottomRight);
	qty:SetText(item.qty);

	local label = Turbine.UI.Label();
	label:SetParent(border);
	label:SetPosition(35,1);
	label:SetSize(border:GetWidth()-87,32);
	label:SetFont(Turbine.UI.Lotro.Font.Verdana16);
	label:SetBackColor(Turbine.UI.Color(0.1,0.1,0.1));
	label:SetOutlineColor(Turbine.UI.Color(0.5,0,0));
	label:SetFontStyle(Turbine.UI.FontStyle.Outline);
	label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	label:SetText(item.desc);
	label:SetMultiline(true);

	local qui = Turbine.UI.Label();
	qui:SetParent(border);
	qui:SetPosition(border:GetWidth()-52,1);
	qui:SetSize(50,32);
	qui:SetFont(Turbine.UI.Lotro.Font.Verdana10);
	qui:SetBackColor(Turbine.UI.Color(0.1,0.1,0.1));
	qui:SetOutlineColor(Turbine.UI.Color(0.5,0,0));
	qui:SetFontStyle(Turbine.UI.FontStyle.Outline);
	qui:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	qui:SetText(item.owner);

	return ctl;
end

-- Efface le contenu de la liste des items avec facilit de garbage
function InventoryPanel:ClearItems()
	-- Efface la liste
	self.ItemBg.List:ClearItems();
end

-- Utilitaire de tri des resultats
function FilterResult_sortByName(i,j)
	return i.desc < j.desc;
end

-- Chargement de la liste des trucs  afficher, selon les parametres de filtres actifs
function InventoryPanel:Load()

	local res = Filter_Search(self.selectedStorage, self.characterSelectionName, self.selectedText);
	
	-- reset de la liste actuelle
	self:ClearItems();
	-- Charge le rsultat
	for i,item in pairs(res) do
		-- Ajoute l'element dans la liste
		self.ItemBg.List:AddItem(self:BuildControlItem(item));
	end
end


function InventoryPanel:Constructor(profile)
	Turbine.UI.Lotro.Window.Constructor(self)
	self:SetText(gLang.WINTITLE);
	self:SetMouseVisible(false);
	-- self:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self:SetSize(400,800);
	self:SetVisible(false);

	-- constantes diverses
	self.font = Turbine.UI.Lotro.Font.Verdana12;
	self.fontColor = Turbine.UI.Color(1,1,1)
	-- create panel controls
	self:AddCharacterSelection();
	self:AddItemFilter();
	self:AddItemList();
	self:AddStorageSelection();
	
	-- par defaut, le filtrage par perso est inactif		
	self:SelectCharacter(gPlayerName);
	-- par defaut, le filtrage est sur le bag
	self:SelectStorage("bag");
	
	-- tracking de la position
	self.PositionChanged = function (sender,args)
		self.settings.top = self:GetTop();
		self.settings.left = self:GetLeft();
	end
	
	-- Recupere les donnes associes
	self.settingsName = profile;
	self.settings = Kiki.Commons.Settings(profile, { top=0; left=0; } );

	-- Positionne sur l'ecran
	self:SetPosition(self.settings.left,self.settings.top);
	-- Force le positionne sur l'ecran
	Kiki.Commons.Onscreen(self);

	
end

function InventoryPanel:DeleteCurrentCharacter()
	local n = self.characterSelectionName;
	if n == gPlayerName then
		Kiki.Commons.Trace("Can't delete current character");
		return;
	end
	gBag.Delete(n);
	gVault.Delete(n);
	gWallet.Delete(n);
	self:BuildMenu();
	self:SelectCharacter(gPlayerName);
end 

function InventoryPanel:Show()
	-- Recharge la liste
	self:Load();
	Kiki.Commons.Onscreen(self);
	self:SetVisible(true);
	self:Activate();
end

function InventoryPanel:Unload()
	self.settings:Save(self.settingsName);
end

gPanel = InventoryPanel(gPluginName.."_panel"); 


